
import numpy as np
import tiledb


class TileDB:
    def __init__(self,account_name=None,account_key=None):
        if None not in (account_key,account_name):
            config = tiledb.Config()
            config["vfs.azure.storage_account_name"] = account_name
            config["vfs.azure.storage_account_key"] = account_key
            config["vfs.azure.use_https"] = "true"
            config["vfs.azure.blob_endpoint"] = f"{account_name}.blob.core.windows.net"
            self.ctx = tiledb.Ctx(config)
        else:
            # upload local
            self.ctx = None
    def create_image_array(self,image_data,img_uri,compression=True):
        img_shape=image_data.shape
        domain = tiledb.Domain(
            tiledb.Dim(
                name="x", domain=(0, img_shape[0] - 1), tile=img_shape[0], dtype=np.int32
            ),
            tiledb.Dim(
                name="y", domain=(0, img_shape[1] - 1), tile=img_shape[1], dtype=np.int32
            ),
            tiledb.Dim(
                name="z", domain=(0, 3- 1), tile=img_shape[2], dtype=np.int32
            ),
        )
        # create multi-component attribute with three int32 components
        if compression:
            attr = tiledb.Attr( filters=tiledb.FilterList([tiledb.GzipFilter()]))
        else:
            attr = tiledb.Attr()
        schema = tiledb.ArraySchema(domain=domain, sparse=False, attrs=[attr])
        if self.ctx is not None:  # if the storage is cloud
            tiledb.Array.create(img_uri, schema,ctx=self.ctx)
        else:  # if the storage is not local means no remote context should be given
            tiledb.Array.create(img_uri, schema)
        image_data_rgb = image_data
        print("uploaded_image shape",image_data_rgb.shape)
        if self.ctx is not None: # if the storage is cloud
            with tiledb.open(img_uri, "w",ctx=self.ctx) as A:
                # write data to 1st image_id slot
                A[:] = image_data_rgb
        else: # if the storage is local
            with tiledb.open(img_uri, "w") as A:
                # write data to 1st image_id slot
                A[:] = image_data_rgb

    def create_video_array(self,video_data,video_uri,compression=True):
        img_shape=video_data.shape
        domain = tiledb.Domain(
            tiledb.Dim(
                name="t", domain=(0, img_shape[0] - 1), tile=4, dtype=np.int32
            ),
            tiledb.Dim(
                name="x", domain=(0, img_shape[1] - 1), tile=img_shape[1], dtype=np.int32
            ),
            tiledb.Dim(
                name="y", domain=(0, img_shape[2]- 1), tile=img_shape[2], dtype=np.int32
            ),
            tiledb.Dim(
                name="z", domain=(0, img_shape[3]- 1), tile=img_shape[3], dtype=np.int32
            ),
        )
        # create multi-component attribute with three int32 components
        if compression:
            attr = tiledb.Attr( filters=tiledb.FilterList([tiledb.GzipFilter()]))
        else:
            attr = tiledb.Attr()

        schema = tiledb.ArraySchema(domain=domain, sparse=False, attrs=[attr])
        if self.ctx is not None:  # if the storage is cloud
            tiledb.Array.create(video_uri, schema,ctx=self.ctx)
        else:  # if the storage is not local means no remote context should be given
            tiledb.Array.create(video_uri, schema)
        if self.ctx is not None: # if the storage is cloud
            with tiledb.open(video_uri, "w",ctx=self.ctx) as A:
                # write data to 1st image_id slot
                A[:] = video_data
        else: # if the storage is local
            with tiledb.open(video_uri, "w") as A:
                # write data to 1st image_id slot
                A[:] = video_data

    def read_image_array(self,img_uri,x=0,y=0,height=0,width=0,need_full=False):
        if self.ctx is not None:
            with tiledb.open(img_uri,ctx=self.ctx) as A:
                if need_full:
                    return A[:]
                return A[x:x+height,y:y+width,:]
        else:
            with tiledb.open(img_uri) as A:
                if need_full:
                    return A[:]
                return A[x:x+height,y:y+width,:]
    def read_video_array(self,video_uri,frame_id):
        if self.ctx is not None:
            with tiledb.open(video_uri,ctx=self.ctx) as A:
                return A[frame_id,:,:,:]
        else:
            with tiledb.open(self.video_uri) as A:
                return A[frame_id,:,:,:]
